/* draaischijf i2c master (voor gebruik met de draaischijf sketch vanaf versie 0.2
 *  en een lcd op I2C adres hex27
 *  (c) Fenna Pel, maart 2020
 *  pereltje@pereltje.nl
 *  
 *  Deze code is "as is" beschikbaar gesteld onder de GPL v3 licentie
 *  zonder garantie voor geschiktheid voor enig doel
 *  
 *  mocht blijken dat delen van de broncode een andere licentie vereisen, geldt die licentie voor die delen van de broncode
 *  
 *  veranderingen voor versie 0.2:
 *  - opschonen code
 *  - leesbaarheid van de broncode vergroten
 */

 // I2C library gebruiken
#include <Wire.h>

// gebruikt delen van deze code: https://www.bitsandparts.nl/documentation/49/I2C_Display_test_20x4.ino
// begin overgenomen code:
#include <LCD.h>
#include <LiquidCrystal_I2C.h>  // F Malpartida's NewLiquidCrystal library
//Download: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
// Move original LiquidCrystal library elsewhere, copy this in it's place

/*-----( Declare Constants )-----*/
#define I2C_ADDR    0x27  // Define I2C Address for the PCF8574T 
//---(Following are the PCF8574 pin assignments to LCD connections )----
// This are different than earlier/different I2C LCD displays
#define BACKLIGHT_PIN  3
#define En_pin  2
#define Rw_pin  1
#define Rs_pin  0
#define D4_pin  4
#define D5_pin  5
#define D6_pin  6
#define D7_pin  7

#define  LED_OFF  1
#define  LED_ON  0

/*-----( Declare objects )-----*/  
LiquidCrystal_I2C  lcd(I2C_ADDR,En_pin,Rw_pin,Rs_pin,D4_pin,D5_pin,D6_pin,D7_pin);
// einde overgenomen code

// inputs: 
const int knop1pin = 10;
const int knop2pin = 9;
const int knop3pin = 8;
const int knop4pin = 7;
const int knop5pin = 6;
const int knop6pin = 5;
const int knop7pin = 4;
const int knoplinkspin = 11;
const int knoprechtspin = 12;
const int toetsvertraging = 250;

// variabelen:
int knop1 = 0;
int knop2 = 0;
int knop3 = 0;
int knop4 = 0;
int knop5 = 0;
int knop6 = 0;
int knop7 = 0;
int knoplinks = 0;
int knoprechts = 0;
int gewenstspoor = 1;
int actiefspoor = 0;
int recoverspoor = gewenstspoor;
int draaischijfgewenstspoor = 0;
int draaischijfactiefspoor = 0;
int draaischijfstatus = 0;


void setup()
{
  pinMode(knop1pin, INPUT_PULLUP);
  pinMode(knop2pin, INPUT_PULLUP);
  pinMode(knop3pin, INPUT_PULLUP);
  pinMode(knop4pin, INPUT_PULLUP);
  pinMode(knop5pin, INPUT_PULLUP);
  pinMode(knop6pin, INPUT_PULLUP);
  pinMode(knop7pin, INPUT_PULLUP);
  pinMode(knoplinkspin, INPUT_PULLUP);
  pinMode(knoprechtspin, INPUT_PULLUP);

  // begin overgenomen code
  lcd.setBacklightPin ( BACKLIGHT_PIN, POSITIVE );
  lcd.setBacklight ( LED_OFF );
  lcd.backlight();
  lcd.begin(20,4);
  // eind overgenomen code
}

void loop()
{
  leesdraaischijf();
  if (draaischijfstatus == 2) {
    draaischijfresetrecover();
    schrijfscherm();
  }
  schrijfscherm();
  knop1 = !digitalRead(knop1pin);
  knop2 = !digitalRead(knop2pin);
  knop3 = !digitalRead(knop3pin);
  knop4 = !digitalRead(knop4pin);
  knop5 = !digitalRead(knop5pin);
  knop6 = !digitalRead(knop6pin);
  knop7 = !digitalRead(knop7pin);
  knoplinks = !digitalRead(knoplinkspin);
  knoprechts = !digitalRead(knoprechtspin);
  gewenstspoor = 0 ;
  if (knop1 == HIGH) gewenstspoor = 1;
  if (knop2 == HIGH) gewenstspoor = 2;
  if (knop3 == HIGH) gewenstspoor = 3;
  if (knop4 == HIGH) gewenstspoor = 4;
  if (knop5 == HIGH) gewenstspoor = 5;
  if (knop6 == HIGH) gewenstspoor = 6;
  if (knop7 == HIGH) gewenstspoor = 7;
  if (knoplinks ==HIGH) gewenstspoor = draaischijfactiefspoor - 1;
  if (knoprechts == HIGH) gewenstspoor = draaischijfactiefspoor + 1;
  if (gewenstspoor != 0 && gewenstspoor != draaischijfactiefspoor) {
    recoverspoor = gewenstspoor;
    stuurspoor(gewenstspoor);
  }
  delay(toetsvertraging);
}

void stuurspoor(int stuurspoorgewenstspoor) {
  Wire.beginTransmission(0x28);
  Wire.write(stuurspoorgewenstspoor);     
  Wire.endTransmission();

}

void leesdraaischijf(){
  Wire.requestFrom(0x28, 3); 
  draaischijfstatus = Wire.read();
  draaischijfgewenstspoor = Wire.read();
  draaischijfactiefspoor = Wire.read();
}

void draaischijfresetrecover () {
  leesdraaischijf();
  while (draaischijfstatus != 0) {
    schrijfscherm(); 
    delay(250);
    leesdraaischijf();
  }
  stuurspoor(recoverspoor);
}

void schrijfscherm() {
  lcd.home ();
  lcd.print ("s: ");
  lcd.print (draaischijfstatus);
  lcd.print (" a: ");
  lcd.print (draaischijfactiefspoor);
  lcd.print (" g: ");
  lcd.print (draaischijfgewenstspoor);
  lcd.print (" r: ");
  lcd.print (recoverspoor);
  lcd.print ("      ");
}
